-- by_field.e by Christian Cuvier aka CChris <ccuvier@free.fr>
-- v1.0 of May 2nd, 2006
--
-- Provides the sort_by_nth_field() global function, which sorts a sequence of records 
-- in increasing order of their n-th field.
-- It also provides safe_sort_by_nth_field, a safer, but slower version.
--
-- Usage: result=sort_by_nth_field(source,field_number,direction)
-- where:
-- source (a sequence)
--     is the sequence to be sorted;
-- field_number (an integer)
--     is an integer, the rank of the field according to which source is to be sorted.
-- direction (an integer)
--     tells whether an increasing sort is desired (direction>=0) or decreasing (direction<0).
-- result is the sorted sequence.
--
-- Notes: 
-- If the sorting field is not defined for any of the records being sorted, the
-- function will crash.
-- If this may happen, then use the safe_sort_by_nth_field() function. It takes the same arguments
-- as sort_by_nth_field(), but will handle cases where the n_th field does not exist on some of 
-- the records in source.
-- A record without an n-th field is considered to be less than any that has the n-th field.
-- The result of comparing two records without an n-th field is undefined.
-- safe_sort_by_nth_field() never crashes, but is slower than sort_by_nth_field().

integer field_by_which_to_sort    
-- will hold the rank of the field used to perform the sorting
-- Set by sort_by_nth_field with its field_number argument prior to any sorting.

function sorter_incr(sequence s1,sequence s2)
-- This is the call_back function that replaces compare() when sorting in increasing order:
-- s1 is less (resp. greater) than s2 if its 'field_by_which_to_sort'-th field is
-- less (resp. greater) than the coreesponding one for s2.
return compare(s1[field_by_which_to_sort],s2[field_by_which_to_sort])
end function

function sorter_decr(sequence s1,sequence s2)
-- This is the call_back function that replaces compare() when sorting in decreasing order:
-- s1 is less (resp. greater) than s2 if its 'field_by_which_to_sort'-th field is
-- greater (resp. less) than the coreesponding one for s2.
return compare(s2[field_by_which_to_sort],s1[field_by_which_to_sort])
end function   

constant sorter_incr_id=routine_id("sorter_incr"),
         sorter_decr_id=routine_id("sorter_decr")
-- custom_sort() needs the routine id of the function used to compare two objects.

include sort.e

global function sort_by_nth_field(sequence s,integer n,integer direction)
integer which_id

-- Sets the parameter for the sorting function, then custom_sort()s using that function
field_by_which_to_sort=n

-- Determines which callback to use according to the sort direction
if direction>=0 then which_id=sorter_incr_id
else which_id=sorter_decr_id
end if

-- Sorts using chosen direction.
return custom_sort(which_id,s)
end function

function safe_sorter_incr(sequence s1,sequence s2)
-- This is the call_back function that replaces compare() when sorting in increasing order:
-- s1 is less (resp. greater) than s2 if its 'field_by_which_to_sort'-th field is
-- less (resp. greater) than the coreesponding one for s2.
integer p1,p2
p1=length(s1)
p2=length(s2)
if p1>=field_by_which_to_sort and p2>=field_by_which_to_sort then
-- Most expected case
    return compare(s1[field_by_which_to_sort],s2[field_by_which_to_sort])
elsif p1<field_by_which_to_sort and p2>=field_by_which_to_sort then
    return -1
elsif p2<field_by_which_to_sort and p1>=field_by_which_to_sort then
    return 1
else -- undefined behaviour
    return compare(s1,s2)
end if
end function

function safe_sorter_decr(sequence s1,sequence s2)
-- This is the call_back function that replaces compare() when sorting in decreasing order:
-- s1 is less (resp. greater) than s2 if its 'field_by_which_to_sort'-th field is
-- greater (resp. less) than the coreesponding one for s2.
integer p1,p2
p1=length(s1)
p2=length(s2)
if p1>=field_by_which_to_sort and p2>=field_by_which_to_sort then
-- Most expected case
    return compare(s2[field_by_which_to_sort],s1[field_by_which_to_sort])
elsif p1<field_by_which_to_sort and p2>=field_by_which_to_sort then
    return 1
elsif p2<field_by_which_to_sort and p1>=field_by_which_to_sort then
    return -1
else -- undefined behaviour
    return compare(s2,s1)
end if
end function

constant safe_sorter_incr_id=routine_id("safe_sorter_incr"),
         safe_sorter_decr_id=routine_id("safe_sorter_decr")
-- custom_sort() needs the routine id of the function used to compare two objects.

include sort.e

global function safe_sort_by_nth_field(sequence s,integer n,integer direction)
integer which_id

-- Sets the parameter for the sorting function, then custom_sort()s using that function
field_by_which_to_sort=n

-- Determines which callback to use according to the sort direction
if direction>=0 then which_id=safe_sorter_incr_id
else which_id=safe_sorter_decr_id
end if

-- Sorts using chosen direction.
return custom_sort(which_id,s)
end function











